/**
 * JORM: an implementation of a generic mapping system for persistent Java
 *       objects. Two mappings are supported: to RDBMS and to binary files.
 * Copyright (C) 2001-2004 France Telecom R&D - INRIA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: jorm-team@objectweb.org
 *
 */
package jorm.rdb;

// Import clauses required by jorm
import org.objectweb.jorm.api.Debug;
import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PAccessorGen;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PBindingCtrl;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PClassMappingCtrl;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.jorm.api.PExceptionNoDSI;
import org.objectweb.jorm.api.PExceptionProtocol;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PMappingStructuresManager;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.api.PStateGraph;
import org.objectweb.jorm.genclass.api.FieldDesc;
import org.objectweb.jorm.genclass.api.GenClassProp;
import org.objectweb.jorm.lib.EmptyPNameIterator;
import org.objectweb.jorm.lib.MultiplePNameIterator;
import org.objectweb.medor.eval.lib.TupleAdapter;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.jorm.naming.api.PNameManager;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.api.PNamingContext;
import org.objectweb.jorm.type.api.PExceptionTyping;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.prefetch.api.PrefetchCache;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.tuple.api.Tuple;
import org.objectweb.medor.tuple.api.TupleCollection;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.LoggerFactory;
//import for the rdb mapper
import org.objectweb.jorm.api.PPrefetchTupleCollection;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.api.RdbAdapterException;
import org.objectweb.jorm.mapper.rdb.adapter.api.JoinedTable;
import org.objectweb.jorm.mapper.rdb.lib.RdbExtentGenInfos;
import org.objectweb.jorm.mapper.rdb.lib.RdbTupleCollection;
import org.objectweb.jorm.mapper.rdb.lib.PClassMappingDecoder;
import org.objectweb.jorm.mapper.rdb.lib.PMapperRdb;
import org.objectweb.jorm.mapper.rdb.lib.RdbPMappingStructuresManager;
import org.objectweb.jorm.mapper.rdb.lib.RdbConnectionWrapper;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbClassMapping;
import org.objectweb.jorm.mapper.rdb.metainfo.RdbMapping;
import org.objectweb.jorm.mapper.rdb.lib.RdbPPolymorphicClass;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.naming.api.PNameGetter;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.eval.lib.SelectEvaluator;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.medor.query.api.QueryLeaf;
import org.objectweb.medor.eval.api.ConnectionResources;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.CalculatedParameterOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.query.rdb.lib.BasicRdbStringQueryLeaf;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.SelectProject;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.datasource.lib.ConnectionFactoryDataStore;
import org.objectweb.medor.expression.lib.BasicBinaryArithmeticOperator;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.eval.prefetch.api.PrefetchBuffer;
import org.objectweb.medor.eval.prefetch.api.IntermediaryPrefetchBuffer;
import org.objectweb.medor.eval.prefetch.lib.IntermediaryPrefetchBufferImpl;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.filter.jorm.lib.EncodePName;
import org.objectweb.medor.filter.jorm.lib.CompositePName;
import org.objectweb.medor.filter.jorm.lib.SinglePName;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.optim.lib.IndexesGenerator;
import org.objectweb.perseus.persistence.api.ConnectionHolder;
import org.objectweb.perseus.persistence.api.WorkingSet;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Collections;
import java.util.Collection;
import java.util.ArrayList;
import java.util.List;
import java.util.NoSuchElementException;
import org.objectweb.jorm.mapper.rdb.lib.RdbPrefetchablePCM;


// Import clauses required by the object BnUserProperty
import jorm.BnUserPropertyAccessor;
import jorm.BnUserPropertyPKPNG;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

public class BnUserPropertyMapping
	extends org.objectweb.jonas_ejb.container.jorm.RdbFactory
	implements PClassMapping, PClassMappingCtrl, Loggable
		, PClassMappingDecoder
		, RdbPrefetchablePCM
	, PNameGetter, jorm.BnUserPropertyPKPNG
    {
	protected static final String MAPPER_NAME = "rdb";
	protected static final String PROJECT_NAME = "jonas";
	//Fields
	protected static final String bnUser_NAME = "bnUser";
	protected static final String id_NAME = "id";
	protected static final String theKey_NAME = "theKey";
	protected static final String theValue_NAME = "theValue";
	//Hidden fields
	protected static final String bnUser_id_NAME = "bnUser_id";

	private final static String[][] SUPER_TYPES =  {};

	/** Fully quailied name of the persitent class */
	protected final static String JORM_CLASS_NAME =
		"jorm.BnUserProperty";
	/** The actual binder which does the binding action */
	private PBinder binder;
	private PNameCoder classPNC;

	/** The meta object of the current jorm class */
	private Class moc;
	/** The mapper linked to this PClassMapping */
	private PMapper mapper;
	public PNameCoder bnUserPnc;
	/** the PClassMapping of the sub type */
	private PClassMapping[] subPCMs = null;

	//variables for the resolve method
	private int beginIndex = 0;
	private QueryTree queryTree = null;
	private FieldOperand[] fieldOperands = null;
	private String[] compositeFieldNames = null;
 	/**
	 * The logger instance used by this XMapping instance
	 * and the XBinding instances
	 */
	public Logger logger;
	public LoggerFactory loggerFactory;

	public BnUserPropertyMapping() {
 	}
	public PNameCoder getBnUserPNameCoder() {
	    return bnUserPnc;
	}

	// IMPLEMENTATION OF METHODS FROM THE PClassMappingCtrl INTERFACE

    public void setPNameCoder(PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setPNameCoder(String fn, PNameCoder pnc)
            throws PException, UnsupportedOperationException {
		if (bnUser_NAME.equals(fn)) {
			bnUserPnc = (PNamingContext) pnc;
			return;
		}
		throw new UnsupportedOperationException("This class does'nt contain field reference: " + fn);
    }

    public void setGenClassMapping(PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
    }

    public void setGenClassMapping(String fn, PClassMapping pcm)
            throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class does not contain references to generic class");
    }

    public void setProjectName(String projectname) {
    }


	// IMPLEMENTATION OF METHODS FROM THE PClassMapping INTERFACE

	public void init(PMappingCallback m, MetaObject metaclass) throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		mapper = m;
		moc = (Class) metaclass;
		//the getPType method inits the ptype if not already done
		PType ptype = mapper.getPTypeSpace().createPType(
				"jorm.BnUserProperty",
				SUPER_TYPES);
		setPType(ptype);
        // PType Assignement to the PNC managing the reference bnUser
        String[][] bnUserICDef = {};
        ptype = mapper.getPTypeSpace().createPType(
            "jorm.BnUser",
            bnUserICDef);
        bnUserPnc.setPType(ptype);
        
         try {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
 		//register the structures and dependencies
		init(mapper.getPMappingStructuresManager());
        } catch (Exception e) {
			if(e instanceof RdbAdapterException)
				throw new PException(e, "Column type computing error: ");
			else
				e.printStackTrace();
		} 
			computeReadQuery();
		try {
			computeExtentQuery();
		} catch (ExpressionException e) {
			throw new PException(e);
		}
		computePrefetchedExtentQuery();
		}

	public void init(PMappingStructuresManager pmsm) throws PException {
		pmsm.declareClass(JORM_CLASS_NAME);
		try {
        RdbPMappingStructuresManager msm = (RdbPMappingStructuresManager) pmsm;
        if (mapper == null) {
	       	mapper = msm.getPMapper();
	    }
	    if (adapter == null) {
			adapter = ((PMapperRdb) mapper).getRdbAdapter();
			if (adapter == null) {
				throw new PException("No rdb adapter found on the mapper: " + mapper);
			}
		}
	    // Initialization of the structures of the class jorm.BnUserProperty
			        	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "thekey", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "BnUser_id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "id", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), true, -1, -1), true, true, true);
	            	msm.addTableColumn(JORM_CLASS_NAME, "BnUserProperty", "thevalue", adapter.getSqlType(PTypeSpace.STRING.getTypeCode(), false, -1, -1), false, false, true);
	    					//dependencies
        	pmsm.addDependency(JORM_CLASS_NAME, "jorm.BnUser");
		} catch(Exception e) {
			throw new PException("Error while registering mapping structures: " + e.getMessage());
		}
		classDefined(pmsm);
	}
	
	protected void classDefined() throws PException {
		//register class in the cluster
        classDefined(mapper.getPMappingStructuresManager());
	}

	public void classDefined(PMappingStructuresManager pmsm) throws PException {
		if (pmsm == null) {
			throw new PException("PMappingStructureManager is null: " + JORM_CLASS_NAME + " cannot be registered in a cluster.");
		}
		//register class in the cluster
        pmsm.classDefined(JORM_CLASS_NAME);
	}
	
	public PMapper getPMapper() {
		return mapper;
	}

	public PBinder getPBinder() {
		return binder;
	}

	public PNameCoder getClassPNameCoder() {
		return classPNC;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains references*/
	public PNameCoder getPNameCoder()
		throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PNameCoder getPNameCoder(String fn)
		throws UnsupportedOperationException {
			if (bnUser_NAME.equals(fn))
				return bnUserPnc;
		throw new UnsupportedOperationException("This class does'nt contain field reference " + fn);
	}

	public String getClassName() {
		return JORM_CLASS_NAME;
	}

	/** This method is used only in the case of the current
	 * class is a gen class and contains another gen class*/
	public PClassMapping getGenClassMapping() throws UnsupportedOperationException {
		throw new UnsupportedOperationException("This class is not a gen class");
	}

	public PClassMapping getGenClassMapping(String fn)
		throws UnsupportedOperationException {
			throw new UnsupportedOperationException("This class does not contain references to generic class");
	}

	public boolean isConform(String mapperName) {
		return mapperName.startsWith(MAPPER_NAME);
	}

	public String getProjectName() {
		return PROJECT_NAME;
	}

	public MetaObject getMetaInfo() {
		return moc;
	}

	public String[][] getSuperClassNames() {
		return SUPER_TYPES;
	}

	public void setPBinder(PBinder b) {
		binder = b;
	}

	public void setClassPNameCoder(PNameCoder pnc) throws PException {
		classPNC = pnc;
	}

	public PBinding createPBinding() throws PException {
		if (Debug.ON && logger!=null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "");
		PBinding binding = new jorm.BnUserPropertyBinding();
		binding.init(this);
		return binding;
	}
	/** @deprecated */
    public Iterator getPNameIterator(Object conn) throws PException {
		return getPNameIterator(conn, false, false, null);
    }

	public PNameIterator getPNameIterator(Object conn,
	                                      boolean withSubType,
	                                      boolean prefetching,
	                                      Object txctx) throws PException {
		PNameIterator it = null;
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String mapperName = mapper.getMapperName();
			if(mapperName.indexOf(".") != -1)
				mapperName = mapperName.substring(0, mapperName.indexOf("."));	
			RdbMapping rdbm = (RdbMapping) mapper.getMetaInfoManager().getClass(this.getClassName()).getClassProject(this.getProjectName()).getMapping(mapperName);
			RdbClassMapping rdbcm = (RdbClassMapping) rdbm.getClassMapping();
			String queryToExecute = "";
			int beginIndex = 0;
				queryToExecute = (prefetching ? prefetchedExtentQuery : extentQuery);
				//System.out.println("query to execute:" + queryToExecute);
				//compute the begining index for the PName: number of fields to select + 1
				beginIndex = 4 + 1;
			PreparedStatement ps = connection.prepareStatement(queryToExecute);
			it = new BnUserPropertyResultIterator(txctx, ps, ps.executeQuery(), binder, prefetching, withSubType, logger, beginIndex, this);
        } catch (SQLException e) {
            throw new PExceptionIO(e, "SQL problem while computing the PName Iterator: class <BnUserProperty>.");
        } catch (MedorException e) {
            throw new PExceptionIO(e, "Medor problem while initializing the PName Iterator: class <BnUserProperty>.");
        }
		return it;
	}

	public PClassMapping[] getSubPCMs() throws PException {
		PType ptype = getPType();
		if (ptype == null) {
			throw new PException("No PType defined for the PClassMapping of the class" + JORM_CLASS_NAME);
		}
		PType[] subTypes = ptype.getSubTypes();
		if (subTypes == null || subTypes.length == 0) {
			//No direct subType are mapped then return null
			return null;
		}
		if (subPCMs == null || subPCMs.length != subTypes.length) {
			//compute the PCM array
			subPCMs = new PClassMapping[subTypes.length];
			for (int i=0; i<subTypes.length; i++) {
				subPCMs[i] = mapper.lookup(subTypes[i].getJormName());
			}
		}
		return subPCMs;
	}
	
	// ADD YO //
	public synchronized PName resolve(Object conn, PName pname) throws PException {
				//mapper rdb but not polymorphic
		return pname;
	 	}
	
	public boolean match(Object obj, boolean intermediaryTuple) throws PException {
		return false;
	}
	
	public PName getDecodedPName(TupleCollection tc, PName pname, boolean intermediaryTuple) throws PException {
		PBinder binder = null;
	//test if the tuple is an instance of this class
	if (this.match(tc, intermediaryTuple)) {
		binder  = this.getPBinder();
	} else {
		//test for all the children
		boolean match = false;
		for (int i = 0; i<subPCMs.length && !match; i++) {
				match = subPCMs[i].match(tc, intermediaryTuple);
				binder = subPCMs[i].getPBinder();
		}
		//if no pcm found, error
		if (!match)
			throw new PException("Problem with Medor while resolving pname on the PCM " + JORM_CLASS_NAME + ". No class found for the pname " + pname.toString() + ".");
	}
			return binder.decodeAbstract(pname.encodeAbstract(), null);
		}
	// END ADD YO //
	
    public void configureRefFields(ReferenceConfigurator rc) throws PException, UnsupportedOperationException {
		bnUserPnc = (PNamingContext) rc.getPNameCoder(JORM_CLASS_NAME, bnUser_NAME, "jorm.BnUser");
    }

	public PType getPType() {
		return  binder.getPType();
	}

	public void setPType(PType pt) {
		binder.setPType(pt);
	}


	public void write(PBinding pb, Object conn, PAccessor pa)
		throws PException {
		BnUserPropertyAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnUserPropertyAccessor)
			paccessor = (BnUserPropertyAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_WRITE);
		if (nextState == PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized WRITE operation, status=" + status);
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
		try {
			String sqlquery;
		if (status == PBinding.LIFECYCLE_DELTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Remove");
//Table BnUserProperty : readOnly=false mustGenerateUpdate=true colocatedTable=false joins=$tab.joins
                sqlquery = deleteBnUserProperty12Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUserProperty]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_NEWTOWRITE) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Insert");
                sqlquery = insertBnUserProperty12Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUserProperty]: problematic SQL query -> {" + sqlquery + "}");
                }
		} else if (status==PBinding.LIFECYCLE_ACTIVEFORIO) {
			if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Update");
                sqlquery = updateBnUserProperty12Table(connection, conn, paccessorGen, paccessor, pb.getPName());
                if (sqlquery != null) {
                    throw new PExceptionIO("Risk to loose modifications to class [BnUserProperty]: problematic SQL query -> {" + sqlquery + "}");
                }
		}
		} catch (java.sql.SQLException se) {
			throw new PExceptionIO(se, "SQL problem while writing the DSI.");
		} catch (java.io.IOException ioe) {
			throw new PExceptionIO(ioe, "Stream IO problem while writing the DSI.");
		} catch (RdbAdapterException rae) {
			throw new PExceptionIO(rae, "SQL problem while writing the DSI.");
		}
		// Changing the current state
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
		read(pb, conn, pa, false);
	}
	
	public void read(PBinding pb, Object conn, PAccessor pa, boolean forUpdate) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Read");
		byte status = pb.getStatus();
		BnUserPropertyAccessor paccessor = null;
		PAccessorGen paccessorGen = null;
		if (pa == null)
			throw new PExceptionProtocol("Accessor must not be null");
		else if (pa instanceof BnUserPropertyAccessor)
			paccessor = (BnUserPropertyAccessor) pa;
		else if (pa instanceof PAccessorGen)
			paccessorGen = (PAccessorGen) pa;
		else
			throw new PExceptionProtocol("Wrong type of accessor: " + pa);
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet resultSet = null;
        PException resex = null;
		try {
            String jormrq = readQuery;
            if (forUpdate) {
            	jormrq = jormrq + " FOR UPDATE";
            }
            pstmt = connection.prepareStatement(jormrq);
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnUserPropertyPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnUserPropertyPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
            resultSet = pstmt.executeQuery();
            if(resultSet.next() == false) {
                throw new PExceptionNoDSI(
                    "No entry matching with the following identifier: " 
                    + pname + ", query=" + jormrq);
            }
            PName refname;
            Object fieldval;
            long notnullval;
			if (paccessor != null) {
				//read public primitive fields
                paccessor.paSetId(adapter.getString(resultSet, 2, null));
                paccessor.paSetTheKey(adapter.getString(resultSet, 3, null));
                paccessor.paSetTheValue(adapter.getString(resultSet, 4, null));
				//decode PName for references
				//writeReadRef  isSpecific=true gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@111e6b9 / cnId=1 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@42a751} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4c72e3 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@5976c2 / nd.nr.cn={id=bnUser_id} / fieldName=bnUser / join=null
                refname = bnUserPnc.decodeAbstract(getInstancePNG1(), resultSet);
                paccessor.paSetBnUser(refname, conn);
			} else if (paccessorGen != null) {
				//read public primitive fields
                paccessorGen.paSetStringField(id_NAME, adapter.getString(resultSet, 2, null));
                paccessorGen.paSetStringField(theKey_NAME, adapter.getString(resultSet, 3, null));
                paccessorGen.paSetStringField(theValue_NAME, adapter.getString(resultSet, 4, null));
				//decode PName for references
				//writeReadRef  isSpecific=false gen_ref=org.objectweb.jorm.mapper.rdb.generator.RdbGenRef@111e6b9 / cnId=1 / cnPackage=jorm / cnFieldColumns={id=org.objectweb.jorm.mapper.rdb.generator.RdbGenColumn@42a751} / refColumn=null / nd=org.objectweb.jorm.metainfo.lib.BasicNameDef@4c72e3 / nd.nr.cn=org.objectweb.jorm.metainfo.lib.BasicCompositeName@5976c2 / nd.nr.cn={id=bnUser_id} / fieldName=bnUser / join=null
                refname = bnUserPnc.decodeAbstract(getInstancePNG1(), resultSet);
                paccessorGen.paSetRefField(bnUser_NAME, refname, conn);
			}
		} catch (PException pe) {
			resex = pe;
		} catch (Exception pe) {
			resex = new PExceptionIO(pe, "Problem while reading the DSI.");
		} finally {
		    try {
		        if (resultSet != null) {
			        resultSet.close();
			        resultSet = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
			    	resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		((PBindingCtrl) pb).setStatus(nextState);
	}
	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx) throws PException {
		read(pb, conn, pa, txctx, false);
	}

	public void read(PBinding pb, Object conn, PAccessor pa, Object txctx, boolean forUpdate) throws PException {
	    PrefetchCache pc = getPMapper().getPrefetchCache();
	    if (pc == null || txctx == null) {
	        read(pb, conn, pa, forUpdate);
	        return;
	    }
		PName pname = pb.getPName();
	    try {
	        Tuple tuple = pc.getPrefetchTuple(pname, this, txctx);
	        if (tuple == null) {
	            read(pb, conn, pa, forUpdate);
	            return;
	        }
		    BnUserPropertyAccessor paccessor = null;
		    PAccessorGen paccessorGen = null;
		    if (pa == null)
			    throw new PExceptionProtocol("Accessor must not be null");
		    else if (pa instanceof BnUserPropertyAccessor)
			    paccessor = (BnUserPropertyAccessor) pa;
		    else if (pa instanceof PAccessorGen)
			    paccessorGen = (PAccessorGen) pa;
		    else
			    throw new PExceptionProtocol("Wrong type of accessor: " + pa);
			byte status = pb.getStatus();
		    byte nextState = PStateGraph.nextStatePBinding(status, PBinding.ACTION_READ);
		    if (nextState==PBinding.LIFECYCLE_ERROR)
			    throw new PExceptionProtocol("Unauthorized READ operation, status=" + status);
            Object fval;
            long notnullval;
            if (paccessor != null) {
                paccessor.paSetBnUser(bnUserPnc.decodeAbstract(getInstancePNGPrefetchbnUser(), tuple), conn);
                paccessor.paSetId(tuple.getString(2));
                paccessor.paSetTheKey(tuple.getString(3));
                paccessor.paSetTheValue(tuple.getString(4));
            } else if (paccessorGen != null) {
                paccessorGen.paSetRefField(bnUser_NAME, bnUserPnc.decodeAbstract(getInstancePNGPrefetchbnUser(), tuple), conn);
                paccessorGen.paSetStringField(id_NAME, tuple.getString(2));
                paccessorGen.paSetStringField(theKey_NAME, tuple.getString(3));
                paccessorGen.paSetStringField(theValue_NAME, tuple.getString(4));
            }
			((PBindingCtrl) pb).setStatus(nextState);
		} catch (MedorException me) {
		    throw new PExceptionProtocol(me, "Problem while trying to prefetch data from Medor");
		}
    }

	public boolean exist(PBinding pb, Object conn) throws PException {
		if (Debug.ON && logger != null && logger.isLoggable(BasicLevel.DEBUG)) logger.log(BasicLevel.DEBUG, "Exist");
		byte status = pb.getStatus();
		byte nextState =
			PStateGraph.nextStatePBinding(status, PBinding.ACTION_EXIST);
		if (nextState==PBinding.LIFECYCLE_ERROR)
			throw new PExceptionProtocol("Unauthorized EXIST operation, status=" + status);
		((PBindingCtrl) pb).setStatus(nextState);
		PName pname = pb.getPName();
		Connection connection = RdbConnectionWrapper.narrow2SQL(conn);
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        PException resex = null;
        boolean exist = false;
		try {
            pstmt = connection.prepareStatement("SELECT BnUserProperty.id FROM BnUserProperty WHERE BnUserProperty.id = ?");
            Object _pngId = pname.encodeAbstract();
            if (_pngId instanceof jorm.BnUserPropertyPKPNG) {
                adapter.setString(pstmt, 1, ((jorm.BnUserPropertyPKPNG) _pngId).pnGetId(null));
            } else if (_pngId instanceof PNameGetter) {
                adapter.setString(pstmt, 1, ((PNameGetter) _pngId).pngetStringField("id", null));
            } else {
                throw new PExceptionNaming("Invalid abstract encoded value for ID PName: " + _pngId);
            }
			rs = pstmt.executeQuery();
            exist = rs.next();
		} catch (java.sql.SQLException se) {
			resex = new PExceptionIO(se, "SQL problem while testing the existence of a DSI.");
		} finally {
		    try {
			    if (rs != null) {
			        rs.close();
			        rs = null;
			    }
			    if (pstmt != null) {
			        pstmt.close();
			        pstmt = null;
			    }
			} catch (SQLException se) {
				if (resex == null) {
					resex = new PExceptionIO(se, "Problem while closing ResultSet or Statment");
			    }
			}
			if (resex != null) {
				throw resex;
			}
		}
		return exist;
	}
    private static BnUserProperty_bnUserPNGPrefetch singletonPNGPrefetchbnUser = null;

    private BnUserProperty_bnUserPNGPrefetch getInstancePNGPrefetchbnUser() throws PException {
        if (singletonPNGPrefetchbnUser==null) {
            singletonPNGPrefetchbnUser = new BnUserProperty_bnUserPNGPrefetch(bnUserPnc);
		}
		return singletonPNGPrefetchbnUser;
	}

	//the map of [pclassmapping->table of int]
	//build dynamically
	private HashMap associationTable =  new HashMap();
	
	public HashMap getAssociationTable(){
		return associationTable;
	}
	
	public void addAssociation(PClassMapping targetClass, int[] indexes){
		associationTable.put(targetClass, indexes);
	}
	
	public int[] getIndexesTable(PClassMapping targetClass){
		return (int[]) associationTable.get(targetClass);
	}
	

    private static jorm.BnUserPKPNG singletonPNG1 = null;
    public jorm.BnUserPKPNG getInstancePNG1() throws PException {
        if (singletonPNG1==null) {
            singletonPNG1 = new  BnUserPropertyPNG1(bnUserPnc, adapter);
		}
		return singletonPNG1;
	}

    private String updateBnUserProperty12Table(
        Connection connection,
        Object conn,
        PAccessorGen paccessorGen,
        BnUserPropertyAccessor paccessor,
    	PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnUserPropertyPKPNG _pngId =
                (jorm.BnUserPropertyPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    bnUserPnc.encodeAbstract(
                        paccessor.paGetBnUser(conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnUserProperty SET thekey = ?, BnUser_id = ?, id = ?, thevalue = ? WHERE id = ?");
				//thekey
                nullField = paccessor.paGetTheKey();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//BnUser_id
                nullField = png1.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//thevalue
                nullField = paccessor.paGetTheValue();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    bnUserPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnUser_NAME, conn));
            pstmt = connection.prepareStatement(res = "UPDATE BnUserProperty SET thekey = ?, BnUser_id = ?, id = ?, thevalue = ? WHERE id = ?");
				//thekey
                nullField = paccessorGen.paGetStringField(theKey_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//BnUser_id
                nullField = png1.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//thevalue
                nullField = paccessorGen.paGetStringField(theValue_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 5, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 5, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String insertBnUserProperty12Table(Connection connection,
                                                Object conn,
                                                PAccessorGen paccessorGen,
                                                BnUserPropertyAccessor paccessor,
    											PName pname)
        throws PException, SQLException, IOException, RdbAdapterException {

		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            jorm.BnUserPropertyPKPNG _pngId =
                (jorm.BnUserPropertyPKPNG)
                pname.encodeAbstract();
            if (paccessor != null) {
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    bnUserPnc.encodeAbstract(
                        paccessor.paGetBnUser(conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnUserProperty (thekey, BnUser_id, id, thevalue) VALUES (?, ?, ?, ?)");
				//thekey
                nullField = paccessor.paGetTheKey();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//BnUser_id
                nullField = png1.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//thevalue
                nullField = paccessor.paGetTheValue();
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
            } else if (paccessorGen != null) {
                jorm.BnUserPKPNG png1 =
                    (jorm.BnUserPKPNG)
                    bnUserPnc.encodeAbstract(
                        paccessorGen.paGetRefField(bnUser_NAME, conn));
                pstmt = connection.prepareStatement(res = "INSERT INTO BnUserProperty (thekey, BnUser_id, id, thevalue) VALUES (?, ?, ?, ?)");
				//thekey
                nullField = paccessorGen.paGetStringField(theKey_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
				//BnUser_id
                nullField = png1.pnGetId(null);
                if (nullField == null) {
                    adapter.setNull(pstmt, 2, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 2, ((java.lang.String) nullField));
                }
				//id
                nullField = paccessorGen.paGetStringField(id_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 3, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 3, ((java.lang.String) nullField));
                }
				//thevalue
                nullField = paccessorGen.paGetStringField(theValue_NAME);
                if (nullField == null) {
                    adapter.setNull(pstmt, 4, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 4, ((java.lang.String) nullField));
                }
            }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }
    private String deleteBnUserProperty12Table(
    	Connection connection,
    	Object conn,
    	PAccessorGen paccessorGen,
    	BnUserPropertyAccessor paccessor,
    	PName pname) throws PException, SQLException, RdbAdapterException {
		Object nullField = null;
        String res = null;
        PreparedStatement pstmt = null;
        try {
            pstmt = connection.prepareStatement(res = "DELETE FROM BnUserProperty WHERE id = ?");
        jorm.BnUserPropertyPKPNG _pngId =
            (jorm.BnUserPropertyPKPNG)
            pname.encodeAbstract();
				//id
                nullField = paccessor.paGetId();
                if (nullField == null) {
                    adapter.setNull(pstmt, 1, adapter.getSqlTypeCode(PTypeSpace.STRING.getTypeCode(), null));
                } else {
                   	adapter.setString(pstmt, 1, ((java.lang.String) nullField));
                }
            if (pstmt.executeUpdate() == 1) {
                res = null;
            }
        } finally {
            if (pstmt != null) {
                pstmt.close();
                pstmt = null;
            }
        }
        return res;
    }

	/**
	 * Is the RdbAdapter to use for managing the various relational database.
	 * This field must not be null after the mapping of the class
	 */
	public RdbAdapter adapter;
	/**
	 * Is the select query to use in the Binding.read method
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The read query is built once time
	 * when the class is mapped.
	 */
	public String readQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String prefetchedExtentQuery = null;

	/**
	 * Is the select query to use in the Mapping.getPNameIterator method
	 * in the case where the prefetching is NOT activated.
	 * The building of the from clause depends on database (outer join syntax).
	 * The RdbAdapter is use to built it. The extent query is built once time
	 * when the class is mapped.
	 */
	public String extentQuery = null;

	/**
	 * Building of the structure defining the FROM clause with
	 *  JoinedTable instances
	 */
	private JoinedTable getJoinedTable() {
		JoinedTable jt = new JoinedTable("BnUserProperty");
		JoinedTable.Join jtj = null;
        return jt;
	}

    public String[] getPrefetchSelectCols() {
		return new String[]{"BnUserProperty.BnUser_id", "BnUserProperty.id", "BnUserProperty.thekey", "BnUserProperty.thevalue"};
	}

	public String getPrefetchWhereClause() {
		return null;
	}

	public java.util.Properties getCNId2Column() {
		java.util.Properties p = new java.util.Properties();
		p.setProperty("id", "BnUserProperty.id");
		return p;
	}

	public List getPrefetchTables() {
		return Collections.singletonList(getJoinedTable());
	}

	/**
	 * Compute the read query with the help of the RdbAdapter
	 */
	public void computeReadQuery() {
		readQuery = adapter.getQuery("BnUserProperty.BnUser_id, BnUserProperty.id, BnUserProperty.thekey, BnUserProperty.thevalue",
			Collections.singletonList(getJoinedTable()),
			"BnUserProperty.id = ?",
			false, false);
	}

	/**
	 * Compute the extent query (without prefetch) with the help of the
	 * RdbAdapter
	 */
	public void computeExtentQuery() throws ExpressionException {
		if (extentQuery == null) {
			extentQuery = adapter.getQuery("BnUserProperty.id",
				Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}
	}

	/**
	 * Compute the extent query (with prefetch) with the help of the RdbAdapter
	 */
	public void computePrefetchedExtentQuery() {
		if (prefetchedExtentQuery == null) {
			prefetchedExtentQuery = adapter.getQuery("BnUserProperty.BnUser_id"+ adapter.getColumnAliasExpr("bnUser_id") + ", BnUserProperty.id"+ adapter.getColumnAliasExpr("id") + ", BnUserProperty.thekey"+ adapter.getColumnAliasExpr("theKey") + ", BnUserProperty.thevalue"+ adapter.getColumnAliasExpr("theValue") + ", BnUserProperty.id" + adapter.getColumnAliasExpr("pk0") + "",
			Collections.singletonList(getJoinedTable()),
			null,
			false, false);
		}

	}


	public PName decode(ResultSet rs, int startindex, Object png) throws IOException, SQLException, PException {
        return classPNC.decodeAbstract(png, null);
	}



	
    // IMPLEMENTATION OF THE SPECIFIC PNAME GETTER
    public java.lang.String pnGetId(Object ctx)
            throws PException {
	    BnUserPropertyAccessor paccessor = null;
		if (!(ctx instanceof BnUserPropertyAccessor)) {
			throw new PExceptionProtocol("Unsupported context type: " + ctx);
		}
        try {
            return ((BnUserPropertyAccessor) ctx).paGetId();
        } catch (Exception e) {
		    throw new PException(e, "Cannot read the field 'id'");
        }
    }
    // IMPLEMENTATION OF THE PNameGetter INTERFACE

    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
		if ("id".equals(fn)) {
			if (ctx instanceof BnUserPropertyAccessor) {
				return ((BnUserPropertyAccessor) ctx).paGetId();
			} else {
			    throw new PExceptionProtocol("Unsupported context type: " + ctx);
			}
        }
        throw new PException("Cannot read the field " + fn);
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PException("Cannot read the field " + fn);
    }
	// IMPLEMENTATION OF METHODS FROM THE Loggable INTERFACE

	public void setLogger(Logger l) {
		logger = l;
	}

	public void setLoggerFactory(LoggerFactory lf) {
		loggerFactory = lf;
	}

	public Logger getLogger() {
		return logger;
	}

	public LoggerFactory getLoggerFactory() {
		return loggerFactory;
	}
}


/**
 * This class permits to a PNamingContext to build a composite PName. One
 * class is generated by NameRef (one by reference field or the class name def).
 * This class implements the generic PNameGetter interface and also the
 * generated pname getter interface of the associated composite name.
 */
class BnUserPropertyPNG1
    implements PNameGetter, jorm.BnUserPKPNG {

    Object fieldval;
    long notnullval;
    public Object png;
    public RdbAdapter adapter;

    public BnUserPropertyPNG1(PNameCoder pnc, RdbAdapter adapter) throws PException {
        png = pnc.getNull().encodeAbstract();
        this.adapter = adapter;
    }

    // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE
    public java.lang.String pnGetId(Object ctx) throws PException {
        if (!(ctx instanceof ResultSet))
            throw new PExceptionTyping("This context is not supported, a java.sql.ResultSet is required.");
        try {

			return adapter.getString(((ResultSet) ctx), 1, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <id>.");
        }
    }

    // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE
    public byte pngetByteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: byte");
    }
    public char pngetCharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: char");
    }
    public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: date");
    }
    public int pngetIntField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: int");
    }
    public long pngetLongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: long");
    }
    public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Byte");
    }
    public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Char");
    }
    public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Int");
    }
    public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Long");
    }
    public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: Short");
    }
    public short pngetShortField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: short");
    }
    public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
        if (!(ctx instanceof ResultSet)) {
            throw new PExceptionTyping(
                    "A java.sql.ResultSet is expected as context, found:" + ctx);
        }
        try {
            if ("id".equals(fn)) {

				return adapter.getString(((ResultSet) ctx), 1, null);
            }
        } catch (Exception e) {
            throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">.");
        }
        throw new PExceptionTyping("No field with the type: string");
    }
    public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: bytearray");
    }
    public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: chararray");
    }
    public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigDecimal");
    }
    public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
        throw new PExceptionTyping("No field with the type: BigInteger");
    }
}

    /**
     * This class permits to a PNamingContext to build a composite PName. One
     * class is generated by NameRef (one by reference field).
     * This class implements the generic PNameGetter interface and also the
     * generated pname getter interface of the associated composite name.
     */
    class BnUserProperty_bnUserPNGPrefetch
        implements PNameGetter, jorm.BnUserPKPNG {
        Object fieldval;
        long notnullval;
        public Object png;

        public BnUserProperty_bnUserPNGPrefetch(PNameCoder pnc) throws PException {
            png = pnc.getNull().encodeAbstract();
        }

        // IMPLEMENTATION OF METHOS FROM THE jorm.BnUserPKPNG INTERFACE

        public java.lang.String pnGetId(Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (((Tuple) ctx).isDefined(1)) {
                    return ((Tuple) ctx).getString(1);
                } else {
                    return ((jorm.BnUserPKPNG) png).pnGetId(null);
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Cannot access PName field <Id>");
            }
        }

        // IMPLEMENTATION OF METHODS FROM THE PNameGetter INTERFACE

        public byte pngetByteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char pngetCharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.util.Date pngetDateField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public int pngetIntField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public long pngetLongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Byte pngetObyteField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Character pngetOcharField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Integer pngetOintField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Long pngetOlongField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.Short pngetOshortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public short pngetShortField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.lang.String pngetStringField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
                if (fn.equals("id")) {
                    if (((Tuple) ctx).isDefined(1)) {
                        return ((Tuple) ctx).getString(1);
                    } else {
                        return ((PNameGetter) png).pngetStringField(fn, null);
                    }
                }
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public byte[] pngetByteArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public char[] pngetCharArrayField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigDecimal pngetBigDecimalField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }

        public java.math.BigInteger pngetBigIntegerField(String fn, Object ctx) throws PException {
            if (!(ctx instanceof Tuple)) {
                throw new PExceptionTyping("This context is not supported, a org.objectweb.medor.tuple.api.Tuple is required.");
            }
            try {
            } catch (Exception e) {
                throw new PExceptionIO(e, "Problem while retreiving PNameGetter field <" + fn + ">");
            }
            throw new PExceptionProtocol("Unknown field in PNameGetter: " + fn);
        }
    }



class BnUserPropertyResultIterator implements PNameIterator, jorm.BnUserPropertyPKPNG {
    ResultSet rs;
	private PBinder binder;
	private BnUserPropertyMapping pclassMapping;
    private boolean hasNext;
    private boolean isForw;
	private PreparedStatement ps;
	private Logger logger;

    private boolean prefetching;
    
    private PPrefetchTupleCollection tc = null;
    private int pnameIndex;

	BnUserPropertyResultIterator(Object txctx,
		PreparedStatement ps,
		ResultSet r,
		PBinder binder,
		boolean prefetching,
		boolean withSubType,
		Logger l,
		int pi,
		BnUserPropertyMapping pcm) throws SQLException, MedorException {

		this.ps = ps;
		this.binder = binder;
		this.prefetching = prefetching;
        rs = r;
        logger = l;
        pclassMapping = pcm;
        if (!prefetching) {
        	pnameIndex = 1;
        } else {
        	pnameIndex = pi;
        	tc = new RdbTupleCollection(txctx, r, ps, pnameIndex, logger, pclassMapping, withSubType, this);
        }
        hasNext = (!prefetching) ? rs.next() : tc.next();
        isForw = true;
    }

	public void finalize() throws Throwable {
		if (rs != null)
			rs.close();
		if (ps != null)
			ps.close();
		super.finalize();
	}

    public void close() {
        if (!prefetching) {
            if (rs!= null) {
                try {
                    rs.close();
                } catch(Exception e) {
                }
                rs = null;
            }
            hasNext = false;
            isForw = false;
        }
    }

    public boolean hasNext() {
    	boolean res;
        if (isForw) {
            res = hasNext;
        } else {
            isForw = true;
            try {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                res = hasNext;
            } catch(Exception e) {
                e.printStackTrace();
                res = false;
            }
        }
        if (res == false) {
        	try {
				if (rs != null) {
					rs.close();
					rs = null;
				}
				if (ps != null) {
					ps.close();
					ps = null;
				}
			} catch (SQLException e) {
				rs = null;
				ps = null;
			}
        }
        return res;
    }

    public void remove() throws UnsupportedOperationException, IllegalStateException {
        throw new UnsupportedOperationException();
    }

    public Object next() throws NoSuchElementException {
        try {
            if (!isForw) {
                hasNext = (!prefetching) ? rs.next() : tc.next();
                isForw = true;
            }
            if (!hasNext) {
                throw new NoSuchElementException();
            }
            isForw = false;
            if (prefetching) {
            	tc.getPrefetchBuffer().addPrefetchTuple();
            }
            return pclassMapping.decode(rs, -1, this);
        } catch (NoSuchElementException e) {
        	throw e;
        } catch (Exception e) {
        	if (logger != null) {
        		logger.log(BasicLevel.ERROR, "Impossible to fetch the next element", e);
        	}
            throw new NoSuchElementException(
                    "Impossible to decode to an ID PName: " + e.getMessage());
        }
    }

    // IMPLEMENTATION OF THE BnUserPKPNG INTERFACE //

    public java.lang.String pnGetId(Object ctx) throws PExceptionIO, PExceptionTyping {
        try {
            return pclassMapping.adapter.getString(rs, pnameIndex + 0, null);
        } catch (Exception e) {
            throw new PExceptionIO(e, "Impossible to fetch a the field id of the pname");
        }
    }
}
